@@ -103,8 +103,9 @@ if yaml:
 
     configuration_providers.append(YAML())
 
-# TODO: add a configuration provider for taking command-line arguments
-# from a file
+# TODO: add configuration providers
+# - for taking command-line arguments from a file
+# - for .ini files
 
 __all__.extend([i.__class__.__name__ for i in configuration_providers])
 
@@ -185,7 +186,6 @@ class ListCLI(BaseCLI):
         # TODO: could use 'extend'
         # - http://hg.mozilla.org/build/mozharness/file/5f44ba08f4be/mozharness/base/config.py#l41
 
-        # TODO: what about nested types?
         kw['action'] = 'append'
         return args, kw
 
@@ -222,8 +222,6 @@ class DictCLI(ListCLI):
             raise AssertionError("Each value must be delimited by '%s': %s" % (self.delimeter, value))
         return value.split(self.delimeter, 1)
 
-# TODO: 'dict'-type cli interface
-
 types = {bool:  BoolCLI(),
          int:   IntCLI(),
          float: FloatCLI(),
@@ -293,7 +291,7 @@ class Configuration(optparse.OptionParser):
     ### TODO: make the class a real iterator
 
     def items(self):
-        # TODO: allow options to be a list of 2-tuples
+        # allow options to be a list of 2-tuples
         if isinstance(self.options, dict):
             return self.options.items()
         return self.options
@@ -315,11 +313,19 @@ class Configuration(optparse.OptionParser):
             _type = self.option_dict[key].get('type')
             if _type is None and 'default' in self.option_dict[key]:
                 _type = type(self.option_dict[key]['default'])
-            if _type is not None and not isinstance(value, _type):
+            if _type is not None:
+                tocast = True
                 try:
-                    config[key] = _type(value)
-                except BaseException, e:
-                    raise TypeCastException("Could not coerce %s, %s, to type %s: %s" % (key, value, _type.__name__, e))
+                    if isinstance(value, _type):
+                        tocast = False
+                except TypeError:
+                    # type is a type-casting function, not a proper type
+                    pass
+                if tocast:
+                    try:
+                        config[key] = _type(value)
+                    except BaseException, e:
+                        raise TypeCastException("Could not coerce %s, %s, to type %s: %s" % (key, value, _type.__name__, e))
 
         return config
 
@@ -410,12 +416,10 @@ class Configuration(optparse.OptionParser):
         for key, value in self.items():
             try:
                 handler = self.types[self.option_type(key)]
-            except:
-                # XXX print this for now
-                # ultimately, if an option can't be coerced to a type
+            except KeyError:
+                # if an option can't be coerced to a type
                 # we should just not add a CLI handler for it
-                print key, value
-                raise
+                continue
             args, kw = handler(key, value)
             if not args:
                 # No CLI interface
